pub mod login;
pub mod game_actions;
pub mod map_actions;

use serde::{Deserialize, Serialize};

use crate::game::chat_message::ChatMessage;

#[derive(Serialize, Deserialize, Default, Debug)]
#[serde(rename_all = "snake_case")]
pub enum Request {
    #[default]
    Error,
    Login(login::LoginRequest),
    Message(ChatMessage),
    GetChatHistory { amount: usize, from: usize },
    Quit,
    Shutdown
}
#[derive(Serialize, Clone)]
#[serde(rename_all = "snake_case")]
pub enum Response {
    Error(RequestError),
    Login(login::LoginResult),
    Message(ChatMessage),
    GetChatHistory(Vec<ChatMessage>),
    Quit { id: String },
    Shutdown,

}

#[derive(Serialize, Debug, Clone)]
#[serde(rename_all = "snake_case")]
pub enum RequestError {
    InvalidRequest,
    AlreadyLoggedIn,
}