use serde::{Deserialize, Serialize};


#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct ChatMessage {
    /// message text, `{item}` can be used to refer to items and such, where item is of the path such as `items/sword` or `spells/fireball`
    pub text: String,
    /// Rolls of the action, if not empty a roll should happen before
    pub roll: Option<Vec<RollDialogOption>>,
    /// Optional roll target
    pub roll_target: Option<i32>,
    /// Optional action buttons, for a chat message this will be empty
    pub actions: Option<Vec<String>>,
    /// Source/Caster/Whoever initiated the action/sent the message
    pub source: String,
    /// Targets of the action, for a chat message this will be empty
    pub targets: Option<Vec<String>>,
    /// message id, should be left emitted or 0 for new messages
    #[serde(default = "default_id")]
    pub id: usize,
}

fn default_id() -> usize { 0 }

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct RollDialogOption {
    /// Field name
    pub name: String,
    /// dice size (aka d6, d12, d20)
    pub dice_type: u16,
    /// amount of dice (aka 1d6, 2d12, 10d20)
    pub dice_amount: u16,
    /// Constant amout to add (+7, +3, -1)
    pub constant: i16,
    /// Extra data, like damage type
    pub extra: String,
    /// should be enabled by default
    pub enabled: bool
}
