use serde::Serialize;

use super::chat_message::ChatMessage;

pub trait GameEntry : Serialize + Sized {
    /// Load a single entry from string
    fn load(entry: &str) -> Option<Self>;
    /// Loads multiple items, blanket impl using Self::load
    fn load_mul(entries: &[&str]) -> Vec<Option<Self>> {
        let mut r = Vec::with_capacity(entries.len());
        for  &e in entries {
            r.push(Self::load(e));
        }
        r 
    }
    /// Display name (e.g. `weapon/dagger` -> `Dagger`)
    fn display_name(&self) -> String;
    /// Get all entries, with an optional filter (could be `weapon` for example to show only weapons)
    fn all(filter: Option<&str>) -> Vec<String>;
    /// Get all categories (such as weapons/consumables/spells)
    fn categories() -> Vec<String>;
    /// returns a chat message to show the entry (with description and all)
    fn to_chat(&self, source: &str) -> ChatMessage;
}

pub struct ActionDefinition {
    pub name: String,
    pub targets: i32,
}

pub struct ActionResult {
    pub name: String,
    pub roll_result: i32,
    pub roll_target: i32,
}