use game::{chat_message::ChatMessage, Game, GameImpl};
use tokio::sync::{broadcast, mpsc};

pub mod user;
pub mod table;
pub mod api;
pub mod game;
pub mod pathfinder2r_impl;

pub struct GameServer {
    _game: Game<pathfinder2r_impl::Pathfinder2rCharacterSheet, pathfinder2r_impl::entry::Entry>,
    chat: Vec<(String, ChatMessage)>,
}
impl GameServer {
    pub fn new() -> Self {
        Self {
            _game: Game::new(),
            chat: Vec::new(),
        }
    }

    pub async fn server_loop(mut self, mut msgs: mpsc::Receiver<(String, api::Request)>, broadcast: broadcast::Sender<api::Response>) {
        while let Some(req) = msgs.recv().await {
            // TODO: do stuff yo!
            let (id, req) = req;
            println!("Got message from {}: {:?}", &id, &req);

            match req {
                api::Request::Error => {},
                api::Request::Login(_) => {},
                api::Request::Message(mut msg) => { 
                    if msg.id == 0 || msg.id >= self.chat.len() {
                        msg.id = self.chat.len() + 1; // set the message id, 0 is invalid
                    }
                    // TODO: check if the editor is an admin as well
                    else if id == self.chat[msg.id].0 {
                        self.chat[msg.id] = (id.clone(), msg.clone());
                    }
                    else {
                        // if its an edit message and editor is not the owner, skip
                        continue;
                    }
                    if msg.source.is_empty() {
                        msg.source = id.clone();
                    }
                    self.chat.push((id.clone(), msg.clone()));
                    _ = broadcast.send(api::Response::Message(msg));
                 },
                api::Request::Quit => { _ = broadcast.send(api::Response::Quit { id })},
                api::Request::Shutdown => todo!(),
                api::Request::GetChatHistory { mut amount, from: last_msg } => {
                    if amount == 0 { amount = self.chat.len(); }
                    let history: Vec<ChatMessage> = self.chat.iter()
                        .skip(last_msg)
                        .take(amount)
                        .map(|m| m.1.clone())
                        .collect();
                    _ = broadcast.send(api::Response::GetChatHistory(history));
                },
            }
        }
        _ = broadcast.send(api::Response::Shutdown);
    }
}